'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global localizePage, BGcall, portInit, chrome, parseUri, translate, Chart */

// the tab object, which contains |id| and |url| (stored as unicodeUrl) of the current tab
let tab = null;

$(() => {
  localizePage();
  const BG = chrome.extension.getBackgroundPage();
  BGcall('recordGeneralMessage', 'popup_opened');

  const ctx = document.getElementById('myChart');

  Chart.pluginService.register({
    beforeDraw(chart) {
      if (chart.config.options.elements.center) {
        // Get ctx from string
        const localCTX = chart.chart.ctx;

        // Get options from the center object in options
        const centerConfig = chart.config.options.elements.center;
        const fontStyle = centerConfig.fontStyle || 'Arial';
        const txt = centerConfig.text;
        const color = centerConfig.color || '#000';
        const sidePadding = centerConfig.sidePadding || 20;
        const sidePaddingCalculated = (sidePadding / 100) * (chart.innerRadius * 2);
        // Start with a base font of 42px
        localCTX.font = `42px ${fontStyle}`;

        // Get the width of the string and
        // also the width of the element minus 10 to give it 5px side padding
        const stringWidth = localCTX.measureText(txt).width;
        const elementWidth = (chart.innerRadius * 2) - sidePaddingCalculated;

        // Find out how much the font can grow in width.
        const widthRatio = elementWidth / stringWidth;
        const newFontSize = Math.floor(30 * widthRatio);
        const elementHeight = (chart.innerRadius * 2);

        // Pick a new font size so it will not be larger than the height of label.
        let fontSizeToUse = Math.min(newFontSize, elementHeight);
        // but don't exceed 56px;
        fontSizeToUse = Math.min(fontSizeToUse, 56);
        // Set font settings to draw it correctly.
        localCTX.textAlign = 'center';
        localCTX.textBaseline = 'middle';
        const centerX = ((chart.chartArea.left + chart.chartArea.right) / 2);
        const centerY = ((chart.chartArea.top + chart.chartArea.bottom) / 2);
        localCTX.font = `${fontSizeToUse}px ${fontStyle}`;
        localCTX.fillStyle = color;

        // Draw text in center
        localCTX.fillText(txt, centerX, centerY);
      }
    },
  });

  chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
    if (!(request.message === 'getCurrentTabInfoResponse')) {
      return;
    }
    // Cache tab object for later use
    const { info } = request;
    ({ tab } = info);

    let host = '';
    if (/^(https?:)/.test(tab.url) === true) {
      ({ host } = parseUri(tab.url));
    }

    if (info.trackingPaused && info.tab_redirect_count === 0) {
      $('#safesiteMsg').text('Tracker Blocking is paused');
    }
    if (info.cdnPaused && info.tab_blocked === 0) {
      $('#safesiteMsg').text('Local Content Caching is paused');
    }

    if (info.trackingPaused) {
      info.tab_blocked = 0;
    }
    if (info.cdnPaused) {
      info.tab_redirect_count = 0;
    }

    if (info.tab_blocked || info.tab_redirect_count || info.tab_HTTPS_redirects) {
      $('#chartDiv').css('display', 'block');
      if (host) {
        $('#domainDiv').text(host);
      }
      /* eslint-disable-next-line no-new */
      new Chart(ctx, {
        type: 'doughnut',
        data: {
          labels: ['Blocked Trackers', 'CDN Redirects',  'Secure URLs'],
          datasets: [{
            data: [info.tab_blocked, info.tab_redirect_count, info.tab_HTTPS_redirects],
            backgroundColor: [
              '#46A4F5',
              '#7289E4',
              '#926BC7',
            ],
          }],
        },
        options: {
          legend: {
            display: false,
          },
          cutoutPercentage: 70,
          maintainAspectRatio: true,
          elements: {
            center: {
              text: `${info.tab_blocked + info.tab_redirect_count + info.tab_HTTPS_redirects}`,
              color: '#000000', // Default black
              fontStyle: 'Helvetica', // Default Arial
              sidePadding: 15, // Default 20 (as a percentage)
            },
          },
        },
      });
      const theChart = document.getElementById('myChart');
      theChart.style.width = '';
    } else {
      if (host) {
        $('#domainDiv').text(host);
        $('#chartDiv').css('display', 'block');
      } else {
        $('#domainDiv').css('display', 'none');
      }
      $('#safetySite').css('display', 'inline');
      $('#safesiteDiv').css('display', 'block');

      $('#myChart').css('display', 'none');
    }
    $('#trackerSwitch').prop('checked', !info.trackingPaused);
    $('#cdnSwitch').prop('checked', !info.cdnPaused);
    $('#secureSwitch').prop('checked', info.secureEnabled);

    sendResponse({});
  }); // end of chrome.runtime.onMessage

  let tabId;
  if (window.location
        && window.location.search) {
    const searchQuery = parseUri.parseSearch(window.location.search);
    if (searchQuery
          && searchQuery.tabId) {
      ({ tabId } = searchQuery);
    }
  }
  if (tabId) {
    chrome.runtime.sendMessage({ message: 'getCurrentTabInfo', tabId });
  } else {
    chrome.runtime.sendMessage({ message: 'getCurrentTabInfo' });
  }

  // Click handlers
  $('#trackerSwitch').change(function trackerClickHandler() {
    BGcall('recordGeneralMessage', 'tracking_paused_clicked');
    BG.adblockTrackingIsPaused(!$(this).is(':checked'));
  });

  $('#cdnSwitch').change(function cdnClickHandler() {
    BGcall('recordGeneralMessage', 'cdn_paused_clicked');
    BG.LocalCDN.adblockCDNIsPaused(!$(this).is(':checked'));
  });

  $('#secureSwitch').change(function cdnClickHandler() {
    BGcall('recordGeneralMessage', 'secure_paused_clicked');
    BG.DDGsettings.updateSetting('httpsEverywhereEnabled', $(this).is(':checked'));
  });

  $('#helpfeedback').click(() => {
    BGcall('recordGeneralMessage', 'help_link_clicked');
    BGcall('openTab', 'https://help.getadblock.com/');
    window.close();
  });
});
