// Inspired by the DuckDuckGo Privacy Extension (https://github.com/duckduckgo/duckduckgo-privacy-extension)
// Original source for this file can be found here:
// https://github.com/duckduckgo/duckduckgo-privacy-extension/tree/develop/shared/js/shared-utils

/*
 * Copyright (C) 2012, 2016 DuckDuckGo, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* For ESLint: List any global identifiers used in this file below */
/* global require log module chrome */

'use strict';

module.exports = (uaStringParam) => {
  let uaString = uaStringParam;
  if (!uaString) {
    uaString = window.navigator.userAgent;
  }

  let browser = '';
  let version = '';

  try {
    const parsedUaParts = uaString.match(/(Firefox|Chrome|Safari)\/([0-9]+)/);
    browser = parsedUaParts[1];
    version = parsedUaParts[2];

    // in Safari, the bit immediately after Safari/ is the Webkit version
    // the *actual* version number is elsewhere
    if (browser === 'Safari') {
      /* eslint-disable-next-line prefer-destructuring */
      version = uaString.match(/Version\/(\d+)/)[1];
    }
  } catch (e) {
    // unlikely, prevent extension from exploding if we don't recognize the UA
    browser = '';
    version = '';
  }

  return {
    browser,
    version,
  };
};
